*
*======================================================================
*
        SUBROUTINE BVALUE
*
* Calculates reduced transition probabilities between bound states
*
*
*   input parameters
*   ================
* 
* AIA    = g.s. spin of particle A (core)
* AIB    = g.s. spin of particle B
* AIC    = g.s. spin of particle C, coupling all momenta of A and B - (channel spin)   
* L0     = orbital angular momentum of the ground state (B orbiting around A)
* AJ0    = angular momentum: AJ0 = |AIB-L0|, ..., AJ0 = |AIB+L0|
* N_1    = nodes of the excited wave function
* L      = orbital angular momentum of the excited state
* AJ     = total angular momentum of the excited state (see explanation of AJ0)
* AICF   = spin of the excited state of C = A + B  - (channel spin)
* JOPT   = 1 (0) sum (does not) over all final angular momenta of C = A + B. 
*          If JOPT=1, the value of AICF can be anything 
* Z1, Z2 = charges of the nuclei
* A1, A2 = masses of the nuclei (in nucleon mass units)
* IOPT   = option for potentials: 1 (2) for Woods-Saxon (M3Y)
* NPTS   = no. of integration points in radial coordinate ( < 10000)
*          (must be the same as in GSWF.INP)
* RMAX   = maximum radius size
*          (must be the same as in GSWF.INP)
* MP     = multipolarity: 0 (M1),  1 (E1),   2 (E2)     
* GA     = magnetic moment (in units of the nuclear magneton) of particle A (core)
* GB     = magnetic moment of particle B
* *** Potential parameters for the excited state:   ***
* V0, R0, AA      = depth, radius, diffuseness of central potential  
* VS0, RS0, AAS   = depth, radius, diffuseness of spin-orbit potential
* RC     = Coulomb radius (usually, RC = R0)  
* FC     = multiplicative factor of central part of M3Y potential
* FSO    = multiplicat. factor of spin-orbit part of M3Y potential
*
      IMPLICIT REAL*8(A-H,O-Z)
*
      PARAMETER(NLG=10000)
*
      COMMON /POTENTIAL/ U(NLG),WSO(NLG),UE(NLG)
     &                   ,VC(NLG)
      COMMON /M3Y/ FC,FSO,UM3Y,VN1
      COMMON /WAVE_FUNCTIONS/ EIGWF(NLG)
      COMMON /PAR/  DR,DR2,PI,FOUR_PI,NPNTS
      COMMON /POT/ V0,VS0,R0,RS0,RC,AA,AAS,Z1Z2,ARED
      COMMON /GF1 / IV
*
      COMPLEX*16 SUMI,SQ(0:2)
      REAL*8 PSIGS(0:NLG),RR(0:NLG),GINT(NLG),ALAM(0:2),EEFF(0:2),
     &    IV(0:130),R(0:NLG),UM3Y(0:NLG),VN1(0:NLG),RU(0:NLG),
     &    WFIN(0:NLG),Y2A(0:NLG),VN2(0:NLG)   
*
      OPEN(UNIT=10,FILE='GSWF.INP',STATUS='OLD')
      OPEN(UNIT=11,FILE='BVALUE.INP',STATUS='OLD')
      OPEN(UNIT=13,FILE='BVALUE.TXT',STATUS='UNKNOWN')
      CALL GFV(30)
*
      WRITE(6,*)' '
      WRITE(13,*)'** B-values for electromagnetic transition i-->f **'
      WRITE(13,*)' '
*
* particle, core, and ground state spins, and angular momentum
      CALL SKIPCOM(11)
      READ(11,*) AIA,AIB,AIC,AJ0,L0
*
      WRITE(13,*) 'Ground state wavefunction spins'
      WRITE(13,*) 'IA      IB      IC      j0        l0'
      WRITE(13,11)AIA,AIB,AIC,AJ0,L0
11    FORMAT(4(F4.1,4X),2X,I2,/)
*
* excited state properties
      CALL SKIPCOM(11)
      READ(11,*) N_1,AJ,L,AICF
*
      WRITE(13,*) 'Excited state wavefunction'
      WRITE(13,*) 'Nodes     j          l         J'
      WRITE(13,12) N_1,AJ,L,AICF
12    FORMAT(I2,7X,F4.1,8X,I2,8X,I2,/)
*
* option for final angular momentum
      CALL SKIPCOM(11)
      READ(11,*) JOPT
*
      WRITE(13,*)'Sum over all final ang. mom.: 1=Y, 0=N'
      WRITE(13,*)' JOPT'
      WRITE(13,19)JOPT
19    FORMAT(2X,I2,/)
*
* charges and masses
      CALL SKIPCOM(11)
      READ(11,*)Z1,A1,Z2,A2
*
* number of integration points and maximum distance
      CALL SKIPCOM(11)
      READ(11,*)IOPT,NPNTS,RMAX
*
* check if you are out of bounds
      IF(NPNTS.GT.NLG .OR. RMAX.GT.250. OR. 
     &       	NPNTS.LT.100 .OR. RMAX.LT.50.) THEN
        WRITE(6,*)'THE INPUT VALUES FOR NPNTS OR RMAX ARE INADEQUATE'
        WRITE(6,*)' 100 <= NPNTS <= 10000,    50 <= RMAX <= 250'
        STOP
      ENDIF
*
      DR=RMAX/NPNTS
*
* potential parameters for the excited state 
      FC=0.
      CALL SKIPCOM(11)
      IF(IOPT.EQ.1) THEN
        READ(11,*)V0,R0,AA,VS0,RS0,AAS,RC
       ELSE
        READ(11,*)FC,FSO,RC      
        DO I=0,NPNTS
          R(I)=I*DR
        ENDDO 
        CALL OMP_READ(R,DR,NPNTS,UM3Y)
        CALL DERIVATIVE(UM3Y,VN1,VN2,DR,NPNTS)             		 
      ENDIF        
*
      WRITE(13,*)'Potential parameters'
      IF(IOPT.EQ.1) THEN
        WRITE(13,*)' V0            R0          AA'
        WRITE(13,20)V0,R0,AA
        WRITE(13,*)' VS0           RS0         AAS          RC'
        WRITE(13,21)VS0,RS0,AAS,RC
       ELSE
        WRITE(13,*)' FC            FSO          RC'
        WRITE(13,20)FC,FSO,RC
      ENDIF
*
      WRITE(13,*)'Charges and masses'
      WRITE(13,*)'Z1     A1     Z2     A2'  
      WRITE(13,22)int(Z1),int(A1),int(Z2),int(A2)
*
20    FORMAT(3(F8.3,4X),/)
21    FORMAT(4(F8.3,4X),/)
22    FORMAT(4(I2,5X),/)
*
      WRITE(13,*)'Radial integration parameters'
      WRITE(13,*)'NPNTS         RMAX'
      WRITE(13,26)NPNTS,RMAX
26    FORMAT(I6,6X,F8.1,/) 
*
* multipolarity
      CALL SKIPCOM(11)
      READ(11,*)MP
*
      WRITE(13,*)'Multipolarity:  0 = M1, 1 = E1, 2 = E2'
      WRITE(13,28)MP
28    FORMAT(2X,I2,/)
*
* magnetic moments
      IF(MP.EQ.0) THEN
        CALL SKIPCOM(11)
        READ(11,*)GA,GB
*
        WRITE(13,*)'Magnetic moments in units of the nuclear magneton'
        WRITE(13,*)' Nucleus A           Nucleus B'
        WRITE(13,29)GA,GB
29      FORMAT(2X,2(G10.4,10X),/)
      ENDIF
*
      CLOSE(UNIT=11)
*
      READ(10,*)NR
      IF (NR.GT.NPNTS) THEN
       WRITE(*,*) '******    STOP: NR larger than NPNTS '
       STOP
      ENDIF
*
* read initial state wavefunction from GSWF.INP
      DO 200 K=1,NR-1
         READ(10,*)RU(K),WFIN(K)
200   CONTINUE
*
      CLOSE(UNIT=10)
*
* interpolate initial wavefunction
      IF(NR.LT.NPNTS) THEN
        DO I=1,NPNTS
          R(I)=I*DR
          X=R(I)
          BIG=1.D30
          CALL SPLINE(RU,WFIN,NR,BIG,BIG,Y2A)
          CALL SPLINT(RU,WFIN,Y2A,NR,X,PSIGS(I))      
        ENDDO
       ELSE
        DO I=1,NPNTS
         PSIGS(I)=WFIN(I)
        ENDDO
      ENDIF
*
      Z1Z2=Z1*Z2
      ARED=A1*A2/(A1+A2)
* famous constants
      PI = 3.14159265359
      FOUR_PI = 4.0D0 *PI
*
* Setup nuclear + coulomb + spin-orbit potential
      CALL POTENT
*
* Integrate Schroedinger equation for excited state
      CALL BOUNDWAVE(N_1,L,AJ,ENER)
*   
      WRITE(13,30)ENER
30    FORMAT(' Excited state energy [MeV]  ',/,1X,D12.4,/)
*
* general parameters and constants
      E2=1.44
      AM0=A1*A2/(A1+A2)
      A13=A1**0.333333+A2**0.333333
      AMU=931.5
      AM=AMU*AM0
      AM2=2.*AM
      HC=197.33
      H2=HC**2
      HMAG=SQRT(E2)*HC/2./AMU
      EEFF(1)=(Z2/A2-Z1/A1)*AM0
      EEFF(2)=(Z2/A2**2+Z1/A1**2)*AM0**2
      EEFF(0)=EEFF(2)
*
* repeated indices for the clebsch and racah coefficients
      AL0=DFLOAT(L0)
      HALF=0.5
      ALAM(0)=1.
      ALAM(1)=1.
      ALAM(2)=2.
      ZERO=0.
      AL0HAT=SQRT(2.*AL0+1.)
      AJ0HAT=SQRT(2.*AJ0+1.)
      AJ0J0=SQRT(AJ0*(AJ0+1.))
      SQ(0)=HMAG*SQRT(3./4./PI)/AL0HAT 
      SQ(1)=SQRT((2.*ALAM(1)+1)*(2.*AJ0+1)/4./PI)
      SQ(2)=SQRT((2.*ALAM(2)+1)*(2.*AJ0+1)/4./PI)
*
* coefficients used in Simpson's integrations
      GINT(1)=DR/3.
      GINT(NPNTS)=DR/3.
      RR(1)=DR
      RR(NPNTS)=NPNTS*DR
      IG=4
      DO 201 K=2,NPNTS-1
        RR(K)=K*DR
        GINT(K)=IG*DR/3.
        IG=6-IG
201   CONTINUE
*
* calculation of reduced transition probabilitiy
      BVAL=0.
      AL=L
      ALL=DBLE(L*(L+1))
      ASS=AIB*(AIB+1.)
      ALS=AJ*(AJ+1.D0)-ALL-ASS
*
* geometric coefficients        
      FACT0=0.
      IF(JOPT.EQ.1) THEN
        DO AF=ABS(AJ-AIA),ABS(AJ+AIA),1.
         FACT0=FACT0+(2*AF+1.)*SIXJ(AJ,AF,AIA,AIC,AJ0,ALAM(MP))**2
        ENDDO
       ELSE
        FACT0=(2.*AICF+1.)*SIXJ(AJ,AICF,AIA,AIC,AJ0,ALAM(MP))**2
      ENDIF
      FACTI=EEFF(MP)*CLEBSH(AJ0,ALAM(MP),AJ,HALF,ZERO,HALF)
*
* integrate over radial coordinates  g.s. --> excited state (Simpson)
      SUMI=0.
      IF(FACTI.NE.0.) THEN
        DO  K=1,NPNTS
           SUMI=SUMI+GINT(K)*EIGWF(K)*PSIGS(K)*(RR(K)**MP)
        ENDDO               
      ENDIF
*
      IF(MP.EQ.0)GOTO 35
*
* Reduced transition probability for E1 or E2 (in units of e^2 fm^2L) 
      SIGN=IV(AL0+AL+ALAM(MP))                ! (-1)**(l+l0+lambda)
      EMAT=(1.+SIGN)/2.*SQ(MP)*FACTI*SUMI
      BVAL=FACT0*ABS(EMAT)**2/E2
      GOTO 40
*   
* M1 transitions (reduced transition probability in units of e^2 fm^2)      
35    IF(AL.NE.AL0)GOTO 40
      AL0P1=AL0+1.
      IF(AJ0.EQ.AL0+0.5)AL0P1=AL0
      IF(AJ.NE.AJ0) THEN
         SIGN=IV(NINT(AL0+0.5-AJ))
        ELSE
         SIGN=0.
      ENDIF
      BML0=SQ(0)*EEFF(0)*(2.*AJ0J0/AL0HAT*AL0P1+SIGN*AJ0HAT/SQRT(2.))
      EMATA=BML0*SUMI
      BML0=0.
      IF(AJ.EQ.AJ0) THEN
         SIGN=IV(NINT(AL0+0.5-AJ0)) 
         BML0=SIGN*SQ(0)*AJ0J0/AL0HAT
        ELSE
         SIGN=-IV(NINT(AL0+0.5-AJ)) 
         BML0=SIGN*SQ(0)*AJ0HAT/SQRT(2.)/AL0HAT
      ENDIF
      EMATB=GB*BML0*SUMI
*
      SIGN=IV(NINT(AIA+AJ0+AICF))              ! (-1)**(Ia+j0+ICf+1)
	EMATC=2.*SQRT((2*AIA+1)*AIA*(AIA+1))*GA*SQ(0)*SIGN*
     &       SIXJ(AIA,AICF,AJ0,AIC,AIA,ALAM(MP))*SUMI*AL0HAT
      EMAT=EMATA+EMATB+EMATC
*
      BVAL=FACT0*ABS(EMAT)**2/E2
*
40    WRITE(13,41)BVAL
41    FORMAT(' Reduced  transition probability [e^2 fm^2L]',/,1X,
     &     D12.4)
*
      WRITE(6,*)'** Output in BVALUE.TXT **'
*
      STOP
      END
**********************************************************************
*    ********    Test input of subroutine BVALUE   *******
*
*  To use the example below, delete the symbol '*' of the corresponding row
*  and copy the module to the corresponding input file.
*
*  The other comment lines should be kept.
*
*=========================================
*
* AIA  =  spin of the particle A (core)
* AIB  =  intrinsic spin of the particle B 
* AIC  =  total angular momentum of the ground state of C = A + B - (channel spin)
* J0   =  single particle angular momentum of B respective to A
* L0   =  relative orbital angular momentum of the ground state
*
* AIA       AIB       AIC     J0      L0
* 
*  1.5       0.5       2       1.5     1 
*
* N_1  =  nodes of the excited state Wave function (exclude origin)   
* J    =  single-particle angular momentum   
* L    =  orbital angular momentum
* AICF =  spin of the excited state after all ang. mom. coupling - (channel spin)
*
* N_1    J        L     AICF 
* 
*  0      1.5      1     1
*
* JOPT = 1 (0) if final state ang. mom., AICF, is (is not) to be
*        summed over all possible values. If JOPT=1, AICF in the
*        previous line can be entered as any value.
*
*  JOPT
*   0 
*
* Z1, Z2  =  charges of the nuclei
* A1, A2  =  masses of the nuclei (in nucleon mass units) 
*
* Z1      A1     Z2     A2
*
*   1.     1.     4.     7.  
*
* IOPT   = option for potentials: 1 (2) for Woods-Saxon (M3Y)
* Integration parameters for radial wavefunctions:
* NPNTS = no. of integration points in radial coordinate ( < 10000)
*         (NPNTS and RMAX must be the same as in GSWF.IN)
* RMAX = maximum radius size (< 250 fm). Same as in GSWF.IN
*
* IOPT    NPTS    RMAX
*
*  1       9999    250.
*
* V0   =  depth of central potential 
* R0   =  radius of the central potential 
* AA   =  diffuseness of the central potential 
* VS0  =  depth of spin-orbit potential 
* RS0  =  radius of the spin-orbit potential 
* AAS  =  diffuseness of the spin-orbit potential 
* RC   =  Coulomb radius (usually, RC = R0)  
*-----------------------------------------------------------------
*  WS  =  V_0 f(r,R0,AA) - V_S0 (l.s) (r_0^2/r) d/dr f(r,RS0,AAS) 
*  
*  f(r,R0,a) = [ 1 + exp((r-R_0)/a) ]^(-1)
*
*  r_0 = 1.4138 fm is the Compton wavelength of the pion. 
*-----------------------------------------------------------------
* V0        R0       AA     VS0     RS0      AAS     RC
*
*  -44.3    2.391    0.52   -9.8    2.391    0.52    2.391 
*  -30.55    2.95     0.52   -8.53    2.95    0.52    2.95 
*
* If IOPT = 2, or else (but not 1), enter FC, FSO and RC:
*    (in this case, insert a '*' sign in above row, or delete it)
* FC      = multiplicative factor of central part of M3Y potential
* FSO     = multiplicat. factor of spin-orbit part of M3Y potential
* RC      =  Coulomb radius   
*
*  FC        FSO       RC
*   1.5       0.2       2.391
*
* MP   = multipolarity: 0 (M1),  1 (E1),   2 (E2)     
*
*  MP
*
*   2
*
* GA  = magnetic moment (in units of the nuclear magneton) of 
*        particle A (core)
* GB  = magnetic moment of particle B
*
*    GA          GB
*
*    2.79        -1.7
* 
**********************************************************************