      SUBROUTINE CONT
*
* Calculates Wavefunctions, S-matrix, and phase-shifts for a 
* Woods-Saxon + Coulomb + spin-orbit optical potential 
*
*  input parameters
*  ================
* 
* IOPT   = option for potentials: 1 (2) for Woods-Saxon (M3Y)
* NPTS   = no. of integration points in radial coordinate ( < 10000)
* RMAX   = maximum radius size
* NEPTS  = number of points in energy ( < 5000)
* V0     = depth of central potential 
* R0     = radius, AA = diffuseness
* VS0, RS0, AAS = depth, radius, diffuseness of spin-orbit potential  
* RC     = Coulomb radius (usually, RC = R0)  
* FC     = multiplicative factor of central part of M3Y potential
* FSO    = multiplicat. factor of spin-orbit part of M3Y potential
* Z1, Z2 = charges of the nuclei
* A1, A2 = masses of the nuclei (in nucleon mass units)
* EI,EF  = Initial energy, final energy
* L      = orbital angular momentum of single particle state 
* J      = total angular momentum of single particle state 
* 
      IMPLICIT REAL*8(A-H,O-Z)
*
      PARAMETER(NLG=10000,NEL=5000)
*
      COMMON/SNBAR/ SNBAR,PSHIFT
      COMMON/DER/AM2,AUX1
      COMMON/INTE1/DR,RI
      COMMON/INTE2/NAX
      COMMON/ET/ETA,XK
      COMMON/POT2/V0,R0,AA,VS0,RS0,AAS,AM,A13,ECM
      COMMON/SPIN1/L
      COMMON/SPIN2/AJ,ALL,ALS
      COMMON/COU/Z1,Z2,A1,A2,RC,RC2,AUX6
      COMMON /M3Y/ FC,FSO,UM3Y,VN1
      COMPLEX*16 PSI(NLG)
      REAL*8 PSH(0:NEL)
      REAL*8 R(0:NLG),UM3Y(0:NLG),VN1(0:NLG),EP(0:NEL),DPSH(0:NEL),
     &    	VN2(0:NLG),DPSH2(0:NEL)
*
      OPEN(UNIT=11,FILE='CONT.INP',STATUS='UNKNOWN')
      OPEN(UNIT=13,FILE='CONT.TXT',STATUS='UNKNOWN')
      OPEN(UNIT=16,FILE='CWAVE.TXT',STATUS='UNKNOWN')
*
      WRITE(6,*)' '
      WRITE(13,*)'** Continuum in cent. + Coulomb + spin-orbit pot **'
      WRITE(13,*)' '
*
      CALL SKIPCOM(11)
      READ(11,*)IOPT,NPNTS,RMAX,NEPTS
*
* check if you are out of bounds
      IF(NPNTS.GT.NLG .OR. RMAX.GT.250. OR. 
     &       	NPNTS.LT.100 .OR. RMAX.LT.50.) THEN
        WRITE(6,*)'THE INPUT VALUES FOR NPNTS OR RMAX ARE INADEQUATE'
        WRITE(6,*)' 100 <= NPNTS <= 10000,    50 <= RMAX <= 250'
        STOP
      ENDIF
*
	DR=RMAX/NPNTS
*
* potential parameters  
      FC=0.
      CALL SKIPCOM(11)
      IF(IOPT.EQ.1) THEN
        READ(11,*)V0,R0,AA,VS0,RS0,AAS,RC
       ELSE
        READ(11,*)FC,FSO,RC      
        DO I=0,NPNTS
          R(I)=I*DR
        ENDDO 
        CALL OMP_READ(R,DR,NPNTS,UM3Y)
        CALL DERIVATIVE(UM3Y,VN1,VN2,DR,NPNTS)             		 
      ENDIF        
*
      CALL SKIPCOM(11)
      READ(11,*)Z1,A1,Z2,A2
*
      WRITE(13,*)'Potential parameters'
      IF(IOPT.EQ.1) THEN
        WRITE(13,*)' V0            R0          AA'
        WRITE(13,20)V0,R0,AA
        WRITE(13,*)' VS0           RS0         AAS          RC'
        WRITE(13,21)VS0,RS0,AAS,RC
       ELSE
        WRITE(13,*)' FC            FSO          RC'
        WRITE(13,20)FC,FSO,RC
      ENDIF
*
      WRITE(13,*)'Charges and masses'
      WRITE(13,*)'Z1     A1     Z2     A2'  
      WRITE(13,22)INT(Z1),INT(A1),INT(Z2),INT(A2)
20    FORMAT(3(F8.3,4X),/)
21    FORMAT(4(F8.3,4X),/)
22    FORMAT(4(I2,5X),/)
*
      CALL SKIPCOM(11)
      READ(11,*)EI,EF,L,AJ
*
      CLOSE(UNIT=11)
*
      WRITE(13,*)'Energy interval and ang. mom. of continuum states'
      WRITE(13,*) 'EI            EF      L        J'
      WRITE(13,25)EI,EF,L,AJ
25    FORMAT(2(F8.2,2X),3X,I2,5X,F4.1,/) 
*
      DE=ABS(EF-EI)/NEPTS
      NAX=NPNTS
*
      WRITE(13,*)'Radial integration parameters'
      WRITE(13,*)'NPNTS         RMAX       NEPTS'
      WRITE(13,26)NPNTS,RMAX,NEPTS
26    FORMAT(I6,5X,F8.1,5X,I6/) 
*
* usual parameters and famous constants
*
      PI=3.141519
      AM0=A1*A2/(A1+A2)
      A13=A1**.333333+A2**.333333
      AM=931.5*A1*A2/(A1+A2)
      AM2=2.*AM
      H2=197.33**2
      RC2=RC*2.
      AUX6=1.44*Z1*Z2
*
* **      Energy mesh   **
*
      IE=0.
      PSH(0)=0.
      IS=0 
      ERESS=0.
      PROD=0.
      IF(EI.EQ.0)EI=1.D-3
      DO 2000 IE=0,NEPTS
	  ECM=EI+IE*DE
        PSH(IE)=0.
        AUX1=AM2*ECM/H2
        XK=DSQRT(AUX1)
        AUX1=AUX1/ECM
        ETA=1.44*Z1*Z2*AM/XK/H2
*
        ALL=DBLE(L*(L+1))
        ALS=AJ*(AJ+1.D0)-ALL-0.75D0
        CALL CONTWAVE(PSI)
        PSH(IE)=PSHIFT  
2000  CONTINUE
*
*
* Look for resonances and change pshift by PI as it crosses a resonance 
      WRITE(13,38)
38    FORMAT('Energy (MeV)',5X,'Phase shift',/)
*
      IR=0 
      DO 3000 IE=2,NEPTS
         ECM=EI+IE*DE
         EP(IE)=ECM
         PROD=PSH(IE)*PSH(IE-1)	
         IF(IE.GT.5.AND.PROD.LT.0.OR.ABS(PSH(IE)-PSH(IE-1)).GT.PI/2) 
     &      THEN
           PSH(IE)=PSH(IE)+PSH(IE-1)/ABS(PSH(IE-1))*PI
           IS=IE-1
           IR=IR+1
           IF(IR.EQ.1)ERESS=EI+IS*DE 
         ENDIF
         WRITE(13,40)ECM,PSH(IE)
3000  CONTINUE
40    FORMAT(F8.3,5X,D12.4)
      PRINT*,' '
*
* **      End energy mesh   **
*
* derivative of phase-shift
      WRITE(13,*)' '
      WRITE(13,50)
50    FORMAT(/,'Energy (MeV)',5X,'Derivative of phase shift',/)
      CALL DERIVATIVE(PSH,DPSH,DPSH2,DE,NEPTS)
      DO IE=2,NEPTS
        WRITE(13,40)EP(IE),DPSH(IE)
      ENDDO
*
* position and width of a resonance
*
      IF(IS.GT.0) THEN
        PRINT*,' |||||  There is at least one resonance |||||'
*
* resonance position (maximum of derivative of phase-shift)
        PRINT*,' Resonance energy=',ERESS,' MeV'
        WRITE(13,60)ERESS
60      FORMAT(/,' Resonance energy=',F8.3,' MeV')
       ELSE
	  PRINT*,' ' 
	  PRINT*,'Structureless continuum: No resonance in this interval' 
      ENDIF
      PRINT*,' '
*
* printout of wavefunction
      PRINT*,'Want output of wavefunction? [1=yes, 0=no]'
      READ(*,*)IRES
      IF(IRES.EQ.1) THEN
        PRINT*,'If yes, enter value of energy [MeV]'
        READ(*,*)ECM
       ELSE
        GOTO 900
      ENDIF                      
*
      AUX1=AM2*ECM/H2
      XK=DSQRT(AUX1)
      AUX1=AUX1/ECM
      ETA=1.44*Z1*Z2*AM/XK/H2
      ALL=DBLE(L*(L+1))
      ALS=AJ*(AJ+1.D0)-ALL-0.75D0
      CALL CONTWAVE(PSI)
      DO I=1,NPNTS
         R(I)=I*DR
         WRITE(16,800)R(I),REAL(PSI(I)),DIMAG(PSI(I))     
      ENDDO
800   FORMAT(3(G10.3,5X))            
*
      CLOSE(UNIT=16)
*
      PRINT*,'- Output of continuum radial wavefunction in CWAVE.TXT'
*
900      PRINT*,'- Output of phase shifts in CONT.TXT'
*
      STOP
      END
**********************************************************************
*    ********    Test input of subroutine CONT  *******
*
*  To use the example below, delete the symbol '*' of the corresponding row
*  and copy the module to the corresponding input file.
*
*  The other comment lines should be kept.
*
*=========================================
*
* IOPT   = option for potentials: 1 (2) for Woods-Saxon (M3Y)
* NPNTS =  no. of integration points in radial coordinate ( < 10000)
* RMAX  =  maximum radius size (< 250 fm)
* NEPTS =  number of points in energy ( < 1000)
*
* IOPT      NPNTS      RMAX      NEPTS
*  
*  1         9999       250.      200
*
* V0  =  depth of central potential     
* VS0 =  depth of spin-orbit potential 
* R0  =  radius of the potential     
* AA  =  diffuseness of the potential 
* RS0 =  radius of the spin-orbit potential  
* AAS =  diffuseness of the spin-orbit potential 
* RC  =  Coulomb radius (usually RC = R0)  
*----------------------------------------------------------------
*  WS =  V_0 f(r,R0,AA) - V_S0 (l.s) (r_0^2/r) d/dr f(r,RS0,AAS) 
*  
*  f(r,R0,A) = [ 1 + exp((r-R0)/a) ]^(-1)
*
*  r_0 = 1.4138 fm is the Compton wavelength of the pion. 
*----------------------------------------------------------------
* V0         R0      AA     VS0     RS0      AAS     RC
*
*  -28.65    2.95      0.52     -8.5      2.95     0.52    2.95 
*  -42.3   2.391    0.52   -9.8    2.391    0.52    2.391 
*
* If IOPT = 2, or else (but not 1), enter FC, FSO and RC:
*    (in this case, insert a '*' sign in above row, or delete it)
* FC      = multiplicative factor of central part of M3Y potential
* FSO     = multiplicat. factor of spin-orbit part of M3Y potential
* RC      =  Coulomb radius   
*
*  FC        FSO       RC
*   1.5       0.2       2.391
*
* Z1, Z2 =  charges of the nuclei
* A1, A2 =  masses of the nuclei (in nucleon mass units) 
*
* Z1    A1    Z2    A2
*
*  1.    1.    4.     7.
*
* EI, EF =  initial energy, final energy, energy step
* L, J   =  orbital angular momentum, angular momentum j (l+s) 
* 
* EI     EF    L      J
*
*  0.     3.    1     1.5     
*
**********************************************************************