*
*======================================================================
*
        SUBROUTINE DICAP
*
* Calculates electromagnetic matrix elements, S-factors, 
* photo-dissociation and radiative capture cross sections
*
*   input parameters
*   ================
* 
* IOPT   = option for potentials: 1 (2) for Woods-Saxon (M3Y)
* NPTS   = no. of integration points in radial coordinate ( < 10000)
*          (must be the same as in GSWF.INP)
* RMAX   = maximum radius size
*          (must be the same as in GSWF.INP)
* NEPTS  = number of points in energy ( < 5000)
* N_0    = nodes of the G.S. wave function
* AIA    = g.s. spin of particle A (core)
* AIB    = g.s. spin of particle B
* AIC    = g.s. spin of particle C, coupling all momenta of A and B - (channel spin)   
* L0     = orbital angular momentum of the ground state (B orbiting around A)
* AJ0    = angular momentum: AJ0 = |AIB-L0|, ..., AJ0 = |AIB+L0|
* EBOUND = binding energy of the ground state
* JOPT   = 1 (0) sum (does not) over all final angular momenta of C = A + B. 
*          If JOPT=1, the value of AICF can be anything 
* AICF   = spin of the excited state of C = A + B - (channel spin)
* Z1, Z2 = charges of the nuclei
* A1, A2 = masses of the nuclei (in nucleon mass units)
* NS,NP1,NP3,ND3,ND5,NF5,NF7 = (1) [0] for  inclusion (no inclusion) of 
*                        s, p, d, and f partial waves
* GA     = magnetic moment (in units of the nuclear magneton) of particle A (core)
* GB     = magnetic moment of particle B
* MP     = multipolarity: 0 (M1),  1 (E1),   2 (E2)     
* SPF    = spectroscopic factor     
* *** potential parameters of the continuum waves:   ***
* V0, R0, AA      = depth, radius, diffuseness of central potential  
* VS0, RS0, AAS   = depth, radius, diffuseness of spin-orbit potential  
* RC     = Coulomb radius (usually, RC = R0)  
* FC     = multiplicative factor of central part of M3Y potential
* FSO    = multiplicat. factor of spin-orbit part of M3Y potential
* EI,EF  = Initial energy, final energy
*
      IMPLICIT REAL*8(A-H,O-Z)
*
      PARAMETER(NLG=10000,NEL=1000)
*
      COMMON/DER/AM2,AUX1
      COMMON/COU/Z1,Z2,A1,A2,RC,RC2,AUX6
      COMMON/INTE1/DR,RI
      COMMON/INTE2/NAX
      COMMON/ET/ETA,XK
      COMMON/POT2/V0,R0,AA,VS0,RS0,AAS,AM,A13,ECM
      COMMON/SPIN1/L
      COMMON/SPIN2/AJ,ALL,ALS
      COMMON /M3Y/ FC,FSO,UM3Y,VN1
      COMMON /GF1 / IV
*
      COMPLEX*16 PSI(NLG),SUMI,SQ(0:2)
      REAL*8 PSIGS(NLG),RR(NLG),GINT(NLG),EMAT(0:6),FPHOT(0:2),
     &      	ALAM(0:2),EEFF(0:2),FACT0(0:6),EP(0:NEL),SFV(0:NEL),
     &        DSFV(0:NEL),DDSFV(0:NEL),RU(0:NLG),WFIN(0:NLG),
     &        Y2A(0:NLG)
      REAL*8 IV(0:130)
      REAL*8 R(0:NLG),UM3Y(0:NLG),VN1(0:NLG),VN2(0:NLG)
      INTEGER*4 NC(0:6)
*
      OPEN(UNIT=10,FILE='GSWF.INP',STATUS='OLD')
      OPEN(UNIT=11,FILE='DICAP.INP',STATUS='OLD')
      OPEN(UNIT=13,FILE='DICAPL.TXT',STATUS='UNKNOWN')
      OPEN(UNIT=14,FILE='DICAP.TXT',STATUS='UNKNOWN')
      OPEN(UNIT=15,FILE='SFAC.TXT',STATUS='UNKNOWN')
*
      CALL GFV(30)
*
      WRITE(6,*)' '
      WRITE(13,*)'** Direct capture cross sections and S-factors  **'
      WRITE(13,*)' '
*
* integration points for distance and energy, maximum distance 
      CALL SKIPCOM(11)
      READ(11,*)IOPT,NPNTS,RMAX,NEPTS
*
      WRITE(13,*)'Radial integration parameters, and # energy points'
      WRITE(13,*)'NPNTS         RMAX       NEPTS'
      WRITE(13,8)NPNTS,RMAX,NEPTS
8     FORMAT(I6,5X,F8.1,5X,I6,/) 
*
* check if we are out of bounds
      IF(NPNTS.GT.NLG .OR. RMAX.GT.250. OR. 
     &       	NPNTS.LT.100 .OR. RMAX.LT.50.) THEN
        WRITE(6,*)'THE INPUT VALUES FOR NPNTS OR RMAX ARE INADEQUATE'
        WRITE(6,*)' 100 <= NPNTS <= 10000,    50 <= RMAX <= 250'
        STOP
      ENDIF
*
	DR=RMAX/NPNTS
      NAX=NPNTS
      IF(MOD(NPNTS,2).EQ.0) NPNTS=NPNTS+1
*
* ground state properties
      CALL SKIPCOM(11)
      READ(11,*) N_0,AIA,AIB,AIC,AJ0,L0,EBOUND
*
      WRITE(13,*) 'Ground state wavefunction spins'
      WRITE(13,*) 'IA      IB      IC      j0        l0'
      WRITE(13,10)AIA,AIB,AIC,AJ0,L0
10    FORMAT(4(F4.1,4X),2X,I2,/)
      WRITE(13,*) 'Ground state energy (MeV)        Nodes'
      WRITE(13,11) EBOUND,N_0
11    FORMAT(F8.3,25X,I2,/)
*
* option for final angular momentum
      CALL SKIPCOM(11)
      READ(11,*) JOPT,AICF
*
c      WRITE(13,*)' '
      WRITE(13,*)'Sum over all final ang. mom.: 1=Y, 0=N'
      WRITE(13,18)JOPT
      WRITE(13,*)'If no, the final angular momentum of A+B is:'
      WRITE(13,19)AICF
18    FORMAT(/,2X,I2,/)
19    FORMAT(2X,F8.3,/)
*
* charges and masses
      CALL SKIPCOM(11)
      READ(11,*)Z1,A1,Z2,A2
*
      WRITE(13,*)'Charges and masses'
      WRITE(13,*)'Z1     A1     Z2     A2'  
      WRITE(13,22)int(Z1),int(A1),int(Z2),int(A2)
22    FORMAT(4(I2,5X),/)
*
* potential parameters for the continuum states 
      FC=0.
      CALL SKIPCOM(11)
      IF(IOPT.EQ.1) THEN
        READ(11,*)V0,R0,AA,VS0,RS0,AAS,RC
       ELSE
        READ(11,*)FC,FSO,RC      
        DO I=0,NPNTS
          R(I)=I*DR
        ENDDO 
        CALL OMP_READ(R,DR,NPNTS,UM3Y)
        CALL DERIVATIVE(UM3Y,VN1,VN2,DR,NPNTS)             		 
      ENDIF        
*
      WRITE(13,*)'Potential parameters'
      IF(IOPT.EQ.1) THEN
        WRITE(13,*)' V0            R0          AA'
        WRITE(13,20)V0,R0,AA
        WRITE(13,*)' VS0           RS0         AAS          RC'
        WRITE(13,21)VS0,RS0,AAS,RC
       ELSE
        WRITE(13,*)' FC            FSO          RC'
        WRITE(13,20)FC,FSO,RC
      ENDIF
20    FORMAT(3(F8.3,4X),/)
21    FORMAT(4(F8.3,4X),/)
*
* initial and final continuum energies
      CALL SKIPCOM(11)
      READ(11,*)EI,EF
*
      WRITE(13,*)' Energy interval of continuum states'
      WRITE(13,*)'   EI           EF'
      WRITE(13,25)EI,EF
25    FORMAT(2(F8.2,4X),/) 
*
      DE=ABS(EF-EI)/NEPTS
*
* partial waves in the calculation 
      CALL SKIPCOM(11)
      READ(11,*)NC(0),NC(1),NC(2),NC(3),NC(4),NC(5),NC(6)
*
      WRITE(13,*)' Partial waves included'
      WRITE(13,*)'S1/2    P1/2    P3/2    D3/2   D5/2    F5/2   F7/2'
      WRITE(13,27)NC(0),NC(1),NC(2),NC(3),NC(4),NC(5),NC(6)
27    FORMAT(7(I2,6X),/)
*
* multipolarity
      CALL SKIPCOM(11)
      READ(11,*)MP,SPF
*
      WRITE(13,*)'Multipolarity:  0 = M1, 1 = E1, 2 = E2'
      WRITE(13,28)MP
      WRITE(13,*)'Spectroscopic factor:'
      WRITE(13,29)SPF
28    FORMAT(2X,I2,/)
29    FORMAT(2X,F9.3,/)
*
* magnetic moments
      IF(MP.EQ.0) THEN
        CALL SKIPCOM(11)
        READ(11,*)GA,GB
*
        WRITE(13,*)'Magnetic moments in units of the nuclear magneton'
        WRITE(13,*)' Nucleus A           Nucleus B'
        WRITE(13,30)GA,GB
30      FORMAT(2X,2(G10.4,10X),/)
      ENDIF
*
      CLOSE(UNIT=11)
*
      READ(10,*)NR
      IF (NR.GT.NPNTS) THEN
       WRITE(*,*) '******    STOP: NR larger than NPNTS '
       STOP
      ENDIF
*
* read initial state wavefunction from GSWF.INP
      DO 200 K=1,NR-1
         READ(10,*)RU(K),WFIN(K)
200   CONTINUE
*
      CLOSE(UNIT=10)
*
* interpolate initial wavefunction
      IF(NR.LT.NPNTS) THEN
        DO I=1,NPNTS
          RR(I)=I*DR
          X=RR(I)
          BIG=1.D30
          CALL SPLINE(RU,WFIN,NR,BIG,BIG,Y2A)
          CALL SPLINT(RU,WFIN,Y2A,NR,X,PSIGS(I))      
        ENDDO
       ELSE
        DO I=1,NPNTS
         PSIGS(I)=WFIN(I)
        ENDDO
      ENDIF
*
* general parameters and famous constants
      E2=1.44
      PI=3.141519
      PI2=2.*PI
      AM0=A1*A2/(A1+A2)
      A13=A1**0.333333+A2**0.333333
      RC2=RC*2.
      AMU=931.5
      AM=AMU*AM0
      AM2=2.*AM
      HC=197.33
      H2=HC**2
      HMAG=SQRT(E2)*HC/2./AMU
      AUX6=E2*Z1*Z2
      EEFF(0)=(Z2/A2+Z1/A1)*AM0
      EEFF(1)=(Z2/A2-Z1/A1)*AM0
      EEFF(2)=(Z2/A2**2+Z1/A1**2)*AM0**2
      H2MC=HC/2./AMU
      AMH2=AM/H2
*
* repeated indices for the clebsh and racah coefficients
      AL0=DFLOAT(L0)
      HALF=0.5
      ALAM(0)=1.
      ALAM(1)=1.
      ALAM(2)=2.
      ZERO=0.
      AL0HAT=SQRT(2.*AL0+1.)
      AJ0HAT=SQRT(2.*AJ0+1.)
      AJ0J0=SQRT(AJ0**2+AJ0)
*
* common factors for strength functions and cross sections 
      SQ(0)=HMAG*SQRT(3./4./PI)/AL0HAT 
      SQ(1)=SQRT((2.*ALAM(1)+1)*(2.*AJ0+1)/4./PI)
      SQ(2)=SQRT((2.*ALAM(2)+1)*(2.*AJ0+1)/4./PI)
      FACE1=3.
      FACE2=15.
      FPHOT(0)=0.
      FPHOT(1)=8.*PI**3*(ALAM(1)+1.)/ALAM(1)/FACE1**2/HC*E2
      FPHOT(2)=8.*PI**3*(ALAM(2)+1.)/ALAM(2)/FACE2**2/HC**3*E2
      FCAP=1./H2*2.*(2.*AIC+1.)/(2.*AIA+1.)/(2.*AIB+1.)
*
* coefficients used in Simpson's integrations
      GINT(1)=DR/3.
      GINT(NPNTS)=DR/3.
      RR(1)=DR
      RR(NPNTS)=NPNTS*DR
      IG=4
      DO 201 K=2,NPNTS-1
        RR(K)=K*DR
        GINT(K)=IG*DR/3.
        IG=6-IG
201   CONTINUE
*
****      start  energy mesh    ***
*
      WRITE(13,*)'   ECM        DB          PHOT         CAP
     &   	      SF'
      WRITE(13,*)'  [MeV]  [e^2 fm^2L/MeV]  [mb]         [mb]
     &         [eV.b]'
      IE=0.
      IF(EI.EQ.0.)EI=1.E-2
      DO 2000 ECM=EI,EF,DE
        IE=IE+1         
        EP(IE)=ECM
        EGAMMA=ECM+EBOUND
        AUX1=AM2*ECM/H2
        XK=DSQRT(AUX1)
        AUX1=AUX1/ECM
        ETA=E2*Z1*Z2*AM/XK/H2        
*
* matrix elements for s, p, d and f waves
*
        IC=0
        DO L=0,3 
          AL=L
          ALL=DBLE(L*(L+1))
          DO AJ=ABS(L-0.5),L+0.5,1.
            EMAT(IC)=0.
            IF(NC(IC).EQ.0)GOTO 3000
            ALS=AJ*(AJ+1.D0)-ALL-0.75D0
*
* geometric coefficients        
            FACT0(IC)=0.
            IF(JOPT.EQ.1) THEN
              DO AF=ABS(AJ-AIA),ABS(AJ+AIA),1.
                FACT0(IC)=FACT0(IC)+(2*AF+1.)*
     &	    		          SIXJ(AJ,AF,AIA,AIC,AJ0,ALAM(MP))**2
              ENDDO
             ELSE
              FACT0(IC)=(2.*AICF+1.)*
     &                	    SIXJ(AJ,AICF,AIA,AIC,AJ0,ALAM(MP))**2
            ENDIF
            FACTI=EEFF(MP)*CLEBSH(AJ0,ALAM(MP),AJ,HALF,ZERO,HALF)
*
* radial integral for g.s. --> l, j continuum (Simpson)        
            CALL CONTWAVE(PSI)
            SUMI=0.
            IF(FACTI.NE.0.) THEN
              DO  K=1,NPNTS
                SUMI=SUMI+GINT(K)*PSI(K)*PSIGS(K)*(RR(K)**MP)
              ENDDO               
            ENDIF
*
            EMAT(IC)=0. 
            IF(MP.EQ.0)GOTO 35
*
* E1 and E2 transitions
            SIGN=IV(AL0+AL+ALAM(MP))                ! (-1)**(l+l0+lambda)
            EMAT(IC)=(1.+SIGN)/2.*SQ(MP)*FACTI*SUMI
            GOTO 3000
*   
* M1 transitions      
35          IF(AL.NE.AL0)GOTO 3000
            AL0P1=AL0+1.
            IF(AJ0.EQ.AL0+0.5)AL0P1=AL0
            IF(AJ.NE.AJ0) THEN
              SIGN=IV(NINT(AL0+0.5-AJ))
             ELSE
              SIGN=0.
            ENDIF
            BML0=SQ(0)*EEFF(0)*( 2.*AJ0J0/AL0HAT*AL0P1+
     &          		  SIGN*AJ0HAT/SQRT(2.) )
            EMATA=BML0*SUMI
*
            BML0=0.
            IF(AJ.EQ.AJ0) THEN
              SIGN=IV(NINT(AL0+0.5-AJ0))               ! (-1)**(l0+1/2-j0)
              BML0=SIGN*SQ(0)*AJ0J0/AL0HAT
             ELSE
              SIGN=-IV(NINT(AL0+0.5-AJ))               ! (-1)**(l0+1/2-j) 
              BML0=SIGN*SQ(0)*AJ0HAT/SQRT(2.)/AL0HAT
            ENDIF
            EMATB=GB*BML0*SUMI
*
            SIGN=IV(NINT(AIA+AJ0+AICF))              ! (-1)**(Ia+j0+ICf+1)
		  EMATC=2.*SQRT((2*AIA+1)*AIA*(AIA+1))*GA*SQ(0)*SIGN*
     &       SIXJ(AIA,AICF,AJ0,AIC,AIA,ALAM(MP))*SUMI*AL0HAT
            EMAT(IC)=EMATA+EMATB+EMATC
*
3000        IC=IC+1
          ENDDO
        ENDDO
*
*  Response function
*
40      DB=0.
        DO IC=0,6
           DB=DB+NC(IC)*AMH2*FACT0(IC)*ABS(EMAT(IC))**2/XK
        ENDDO 
*
* Photo-absorption cross section
*     
        PHOT=SPF*FPHOT(MP)*EGAMMA**(2*MP-1)*DB*10.
        IF(MP.EQ.0)PHOT=sF*FPHOT(1)*EGAMMA*DB*10.
*
* Radiative capture cross section
*
        CAP=EGAMMA**2*FCAP/XK**2*PHOT
*
* S-factor
*
        SF=ECM*CAP*EXP(PI2*ETA)*1.D3
        SFV(IE)=SF
        NTOT=IE
*
4000    WRITE(13,41)ECM,DB,PHOT,CAP,SF
        WRITE(14,41)ECM,DB,PHOT,CAP,SF
41      FORMAT(F8.3,1X,4(D12.4,1X))
2000  CONTINUE
*
      CLOSE(UNIT=14)
*
      CALL DERIVATIVE(SFV,DSFV,DDSFV,DE,NTOT)
      DO I=1,NTOT    
        WRITE(15,41)EP(I),SFV(I),DSFV(I),DDSFV(I)
      ENDDO
*
      CLOSE(UNIT=15)
*
****    End energy mesh         ***
*
      PRINT*,'Output:'
      PRINT*,'DICAP.TXT  - Output of strength function  (2nd column),'
      PRINT*,'             Photodissociation X-section  (3rd column),'
      PRINT*,'             Direct capture cross section (4th column),'
      PRINT*,'             S-factor in eV.b             (5th column)'
      PRINT*,'DICAPL.TXT - Same. But only the table with numbers'
      PRINT*,'SFAC.TXT -   Output of S-Factor (2nd column)'
      PRINT*,'             First derivative   (3rd column)'
      PRINT*,'             Second  derivative (4th column)'
*
      STOP
      END
**********************************************************************
*   ********    Test input of program DICAP  *******
*
*  To use the example below, delete the symbol '*' of the corresponding row
*  and copy the module to the corresponding input file.
*
*  The other comment lines should be kept.
*
*=========================================
* IOPT   = option for potentials: 1 (2) for Woods-Saxon (M3Y)
* NPNTS = no. of integration points in radial coordinate ( < 10000)
* RMAX = maximum radius size (< 250 fm). 
* NEPTS = number of points in energy ( < 1000)
*
* IOPT    NPNTS    RMAX     NEPTS
*  
*  1       9999     250.      200
*
* N_0  =  nodes of the G.S. wave function 
* AIA  =  spin of the particle A (core)
* AIB  =  intrinsic spin of the particle B 
* AIC  =  total angular momentum of the ground state of C = A + B - (channel spin)
* J0   =  single-particle angular momentum 
* L0   =  orbital angular momentum
* EBOUND = binding energy of the ground state (absolute value)
*
* N_0    AIA     AIB     AIC     AJ0      L_0      EBOUND
*
*  0      1.5      0.5    2       1.5      1        0.14
*
* JOPT = 1 (0) if final state ang. mom., AICF, is (is not) to be
*        summed over all possible values. If JOPT=1, AICF  can be 
*        entered as any value.
* AICF =  spin of the excited state after all ang. mom. coupling
*
* JOPT     AICF
*
*  1        1.
*
* Z1, Z2 = charges of the nuclei
* A1, A2 = masses of the nuclei (in nucleon mass units) 
*
* Z1      A1     Z2     A2
*
*   1.     1.     4.     7.  
*  
* V0   =  depth of central potential 
* R0   =  radius of the central potential 
* AA   =  diffuseness of the central potential 
* VS0  =  depth of spin-orbit potential 
* RS0  =  radius of the spin-orbit potential 
* AAS  =  diffuseness of the spin-orbit potential 
* RC   =  Coulomb radius (usually, RC = R0)  
*-----------------------------------------------------------------
*  WS  =  V_0 f(r,R0,AA) - V_S0 (l.s) (r_0^2/r) d/dr f(r,RS0,AAS) 
*  
*  f(r,R0,a) = [ 1 + exp((r-R_0)/a) ]^(-1)
*
*  r_0 = 1.4138 fm is the Compton wavelength of the pion. 
*-----------------------------------------------------------------
* V0        R0       AA     VS0     RS0      AAS     RC
*
*  -44.658   2.391    0.52   -9.8    2.391    0.52    2.391 
*  -45.6    2.391      0.52     -9.8      2.391     0.52    2.391 
*  -28.65    2.95      0.52     -8.5      2.95     0.52    2.95 
*
* If IOPT = 2, or else (but not 1), enter FC, FSO and RC:
*    (in this case, insert a '*' sign in above row, or delete it)
* FC      = multiplicative factor of central part of M3Y potential
* FSO     = multiplicat. factor of spin-orbit part of M3Y potential
* RC      =  Coulomb radius   
*
*  FC        FSO       RC
*   1.5       0.2       2.391
*
* EI,EF =  initial relative energy, final relative energy
* 
* EI       EF    
*
*  0.       3.   
*
* NS1,NP1,NP3,ND3,ND5,NF5,NF7 = (1) [0] for  inclusion (no inclusion) 
*     of  s1/2, p1/2, p3/2, d3/2, d5/2, f5/2, and f7/2  partial waves
*
*  NS     NP1     NP3     ND3    ND5    NF5    NF7
*
*   1       0       0       1      1      0      0
*
* MP   = multipolarity: 0 (M1),  1 (E1),   2 (E2) 
* SF   = Spectroscopic factor    
*
*  MP        SF
*
*   1         1.
*
* GA  = magnetic moment (in units of the nuclear magneton) of 
*        particle A (core)
* GB  = magnetic moment of particle B (proton, neutron, alpha, etc.)
*
*   GA         GB
*
*    -1.7       5.58
* 
**********************************************************************