      PROGRAM RADCAP
*
*  A list of test inputs is given at the end of each subroutines OMP_M3Y.FOR,
*  EIGEN.FOR, BVALUE.FOR. CONT.FOR, and DICAP.FOR, which all part of RADCAP. 
*  
*  C.A. Bertulani, version 2003
*
*--------------------------------------------
*  All dimensional input units are fm and MeV 
*--------------------------------------------    
*
      PRINT*,' Enter:'
      PRINT*,' 1 for M3Y Potential'
      PRINT*,' 2 for energy and wavefunction bound states'
      PRINT*,' 3 for reduced transition probab. between bound states'
      PRINT*,' 4 for phase-shifts and wavefunctions of continuum stat.'
      PRINT*,' 5 for S-factors, response functions, etc.'
      READ(*,*) IOP 
*
      IF(IOP.EQ.1) THEN
        CALL OMP_M3Y
        STOP
      ENDIF
      IF(IOP.EQ.2) THEN
	  CALL EIGEN
        STOP
      ENDIF
      IF(IOP.EQ.3) THEN
	  CALL BVALUE
        STOP
      ENDIF
      IF(IOP.EQ.4) THEN
	  CALL CONT
        STOP
      ENDIF
      IF(IOP.EQ.5) THEN
	  CALL DICAP
        STOP
      ENDIF
      END
*
*======================================================================
*
      SUBROUTINE BOUNDWAVE(NODE_0,L0,AJ,E)
*
      PARAMETER(NLG=10000)
*
* This subroutine solves the Schroedinger equation for bound states
*
      IMPLICIT REAL*8 (A-H,O-Z)
*
      COMMON /WAVE_FUNCTIONS/ EIGWF(NLG)
      COMMON /POTENTIAL/ U(NLG),WSO(NLG),UE(NLG)
     &                   ,VC(NLG)
      DIMENSION       WF0(NLG),
     &                VR(NLG),VE(NLG)
      COMMON /PAR/    DR,DR2,PI,FOUR_PI,NPNTS 
*
      NODE = NODE_0
      L    = L0
*
*  Find a good initial estimate of the eigenvalue
*
      IITER = 0
      NITER = 0
      INODE = NODE
      DEM    = 0.5D0
      COR0  = 0.5D0
      IF(L.EQ.0) COR0 =0.75D0
      COR   =   COR0+1.0D0/(DFLOAT(INODE)+1.0D0)/24.0D0
     *         -1.0D0/(DFLOAT(INODE)+1.0D0)**2/48.0D0
      A12   =    1.0D0/12.0D0
      ACCUR = 0.1D0
      ACCUR0= 0.1D-8
*
* find bottom of the well
*
      AL  = DFLOAT( L*(L+1) )
      ALS = AJ*(AJ+1.0D0)-AL-0.75D0
      EMIN = 0.0D0
      DO IR=1,NPNTS
        XR = IR
        VE(IR) = UE(IR)
        VR(IR) = U(IR) + AL/IR**2 + ALS*WSO(IR)
        IF( EMIN .GT. ( VR(IR)/VE(IR) ) ) EMIN = VR(IR)/VE(IR)
      ENDDO
      IF( NITER. GT. 6 ) GO TO 100
      IF( E .LE. EMIN  .OR.  E .GE. 0.0D0 ) E = EMIN*0.5D0
*
* find turning point and WKB eigen-energy
9     NITER=NITER+1
      S=0.0D0
      S1=0.0D0
      DO 1 IR=1,NPNTS
        P2 = E*VE(IR)-VR(IR)
        IF ( P2.LE. 0.0D0) GO TO 1
        P  = DSQRT( P2 )
        S  = S+P
        S1 = S1+VE(IR)/P
        ITURN = IR
1     CONTINUE
      S = S-PI*( DFLOAT(INODE)+COR )
      DE=-S/S1*2.0D0
108   IF( (E+DE) .GT. EMIN .AND. (E+DE) .LT. 0.0D0 )  GO TO 109
      DE=DE*0.5D0
      GO TO 108
109   E=E+DE
      IF( NITER .GT. 50 ) GO TO 1000
      IF( DABS(DE) .GT. ACCUR ) GO TO 9
      XR=ITURN*DR
100   IITER=IITER+1
110   CONTINUE
*
*  Integrate outward from origin to the turning point
*
      A     = 0.0D0
      B     = 1.0D-10
      WF0(1) = B
      D     = VR(1)-VE(1)*E
      B     = B*(1.0D0-A12*D)
      SEI   = WF0(1)**2*VE(1)
      ICHECK=0
      DO 101 IR=2,ITURN
        C   = 2.0D0*B-A+WF0(IR-1)*D
        A   = B
        B   = C
        D   = VR(IR)-VE(IR)*E
        C   = C/(1.0D0-A12*D)
        SEI = SEI+C**2*VE(IR)
        WF0(IR) = C
        IF(IR.LT.5)  GO TO 101
        IF( WF0(IR)*WF0(IR-1) .LT. 0.0D0 .OR. WF0(IR) .EQ. 0.0D0)
     &   ICHECK=ICHECK+1
101   CONTINUE
      IF(ICHECK.GT.INODE) THEN
        E=E*1.2D0
        DO 1200 IR=1,NPNTS
          IF( VE(IR)*E .GT. VR(IR) ) ITURN=IR
1200    CONTINUE
        GO TO 110
      ENDIF
      IF(ICHECK.LT.INODE) THEN
        E=E*0.9D0
        DO 1201 IR=1,NPNTS
          IF( VE(IR)*E .GT. VR(IR)) ITURN=IR
1201    CONTINUE
        GO TO 110
      ENDIF
      SEI = SEI/C**2-0.5D0*VE(ITURN)
      WFI = C
      WF1I=(274.0D0*WF0(ITURN)-600.0D0*WF0(ITURN-1)
     &  +600.0D0*WF0(ITURN-2)
     &    -400.0D0*WF0(ITURN-3)+150.0D0*WF0(ITURN-4)
     &      -24.0D0*WF0(ITURN-5))/(120.0D0*DR)
      DWFI = WF1I/WFI
      SEI = SEI/DR+A12*DWFI*DR**2
*
*   Integrate inward from infinity to the turning point
*
      D = VR(NPNTS)-VE(NPNTS)*E
      A = DEXP(-DFLOAT(NPNTS)*DSQRT(DABS(D)))
      IF ( A .LT. 1.0D-12) A =0.0D0
      WF0(NPNTS) = A
      A = A*(1.0D0-A12*D)
      D = VR(NPNTS-1)-VE(NPNTS-1)*E
      B = DEXP(-DFLOAT(NPNTS-1)*DSQRT(DABS(D)))
      IF ( B .LE. 1.0D-12 ) B = 1.0D-12
      WF0(NPNTS-1) = B
      SEO = B**2*VE(NPNTS-1)
      B = B*(1.0D0-A12*D)
      DO 201 IR=NPNTS-2,ITURN,-1
        C   = 2.0D0*B-A+WF0(IR+1)*D
        A   = B
        B   = C
        D   = VR(IR)-VE(IR)*E
        C   = C/(1.0D0-A12*D)
        SEO = SEO+C**2*VE(IR)
        WF0(IR)=C
201   CONTINUE
      WFO = C
      WF1O=(-274.0D0*WF0(ITURN)+600.0D0*WF0(ITURN+1)
     *  -600.0D0*WF0(ITURN+2)
     *   +400.0D0*WF0(ITURN+3)-150.0D0*WF0(ITURN+4)
     *     +24.0D0*WF0(ITURN+5))/(120.0D0*DR)
      DWFO = WF1O/WFO
      SEO = SEO/WFO**2-0.50D0*VE(ITURN)
      SEO = SEO/DR-A12*DWFO*DR**2
      SE  = DWFI-DWFO
*
*  Compute the correction to the eigenvalue
*
      DE = SE/(SEI+SEO)
221   IF( DABS(DE) .LT. DEM ) GO TO 222
      DE = DE*0.8D0
      GO TO 221
222   E1 = E+DE
      IF( E1 .GE. 0.0D0 .OR. E1 .LE. EMIN ) THEN
        DE = DE*0.9D0
        GO TO 222
      ENDIF
      E = E1
      IF( IITER .GT. 50 ) GO TO 1000
      XTEST = DABS(DE)/(DABS(E)+1.D-25)
      IF( XTEST .GT. ACCUR0 ) GO TO 100
*
*   Match the wavefunction
*
      SE = WFI/WFO
      DO 303 IR=ITURN,NPNTS
303   WF0(IR)=WF0(IR)*SE
*
*   Normalize the waveunction
*
      SN = 0.0D0
      DX = DR/15.0D0
      AX = DX*7.0D0
      NUP= NPNTS
      IF( MOD(NPNTS,2) .NE.0 ) NUP=NPNTS-1
      DO 301 IR=NUP,1,-1
        SN = SN+WF0(IR)**2*VE(IR)*AX
        AX = DR+DX
        DX =-DX
301   CONTINUE
      SN=1.0D0/DSQRT(SN)
      DO 302 IR=1,NPNTS
        AX = WF0(IR)*SN*DSQRT(VE(IR))
        XR = IR*DR
      WF0(IR) = AX/XR
302   EIGWF(IR) = AX
      RETURN
1000  WRITE(6,*) 'The iteration procces failed  NITER=',NITER,'
     &    IITER='
     *     ,IITER,'  NODE=',NODE,'  E=',E
*
      STOP
      END
*
*======================================================================
*
      SUBROUTINE POTENT
*
      PARAMETER(NLG=10000)
*
*  Construct the potentials for bound state calculations
*
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 UM3Y(0:NLG),VN1(0:NLG)
*
      COMMON /POTENTIAL/ U(NLG),WSO(NLG),UE(NLG)
     &                   ,VC(NLG)
      COMMON /M3Y/ FC,FSO,UM3Y,VN1
      COMMON /PAR/     DR,DR2,PI,FOUR_PI,NPNTS 
      COMMON /POT/ V0,VS0,R0,RS0,RC,AA,AAS,Z1Z2,ARED
*
      NCOUNT=0
*
* Construct the Coulomb potential
*
      DR2 = DR**2
      B_N = 938.9D0*ARED
      B_MASS= (197.329D0)**2/(2.0D0*B_N)
      CHARGE = 1.439976D0*Z1Z2
       DO IR=1,NPNTS
          XR = IR*DR
          XRI=(XR-R0)/AA
          IF(XRI.GT.30.0D0)XRI=30.0D0
          EXT=DEXP( XRI )
*
          XRIS=(XR-RS0)/AAS
          IF(XRIS.GT.30.0D0)XRIS=30.0D0
          EXTS=DEXP( XRIS )
*
*  Central
          U(IR) = V0/( 1.0D0+EXT )
     &                  /B_MASS*DR2
          IF(FC.NE.0.) U(IR) = FC*UM3Y(IR)/B_MASS*DR2
*
*  Spin-orbit 
          RCOMP=1.4138     ! pion Compton wavelength
          IF(FC.EQ.0.) THEN
             WSO(IR) = VS0*RCOMP**2*EXTS/((1.0D0+EXTS)**2*XR*AAS)
     &                  /B_MASS*DR2/2.
            ELSE
             IF(FSO.NE.0.) THEN
              WSO(IR) = - FSO*RCOMP**2/XR*VN1(IR)/B_MASS*DR2/2.
             ENDIF
          ENDIF
*
*  Scaling parameter
          UE(IR) = DR2/B_MASS
*
*  Coulomb potential of a uniformly distributed charge
          VC(IR)=CHARGE/XR
          IF(XR.LT.R0) VC(IR)=CHARGE/R0*
     &      (1.5D0-0.5D0*(XR/R0)**2)
          VC(IR)=VC(IR)*DR2/B_MASS
*
*  add to WS potential
          U(IR)=U(IR)+VC(IR)
      ENDDO
*
      RETURN
      END
*
*======================================================================
*
      FUNCTION VNR(R)
*
* Real part of the potential + spin-orbit potential for continuum 
* states
*
      PARAMETER(NLG=10000)
*
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/POT2/V0,R0,AA,VS0,RS0,AAS,AM,A13,ECM
      COMMON/SPIN2/AJ,ALL,ALS
      COMMON /M3Y/ FC,FSO,UM3Y,VN1
      COMMON/INTE1/DR,RI
      REAL*8 UM3Y(0:NLG),VN1(0:NLG)
*
      VNR=0.
      CR=1.E15
      CRS=1.E15
      TEMP=(R-R0)/AA
      TEMPS=(R-RS0)/AAS
      IF(TEMP.LT.30.)CR=DEXP(TEMP)
      IF(TEMPS.LT.30.)CRS=DEXP(TEMPS)
      VNR=V0/(1.+CR)
      IR=0
      IF(R.GT.0.)IR=NINT(R/DR)
      IF(FC.NE.0.) VNR = FC*UM3Y(IR)
*
*  add spin-orbit term
      RCOMP=1.4138 
      DVNRS=CRS/(1.+CRS)**2/AAS 
      IF(FC.EQ.0.) THEN
         IF(R.GT.1.D-30) WSO=VS0*ALS*RCOMP**2/2./R*DVNRS
        ELSE
         IF(FSO.NE.0.) THEN
          IF(R.GT.1.D-30) WSO = - FSO*ALS*RCOMP**2/R*VN1(IR)/2.
         ENDIF
      ENDIF
      VNR=VNR+WSO
*
      RETURN
      END
*
*======================================================================
*
      FUNCTION VCOU(R)
*
* Coulomb potential of an extended charge distribution of radius RC
* Used in calculation of continuum states
*
      IMPLICIT REAL*8(A-H,O-Z)
      COMMON/COU/Z1,Z2,A1,A2,RC,RC2,AUX6
*
      IF(R.GT.RC)GO TO 1
      VCOU=AUX6*(3.-(R/RC)**2)/RC2
      RETURN
1     VCOU=AUX6/R
*
      RETURN
      END
*
*=====================================================================
*
      SUBROUTINE DENS_READ(R,NPNTS,RHO1,RHO2)
*
* reads densities from file 'DENS.INP': R  x  DENS1(R)  x   DENS2(R) 
* First row in DENS.INP = number of rows in input
*
      PARAMETER(NLG=10000)
*
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 R(0:NLG),RU(0:NLG),Y2A(0:NLG)
      REAL*8 RHO1(0:NLG),RHO2(0:NLG),RH1(0:NLG),RH2(0:NLG)
*
      OPEN(UNIT=52,FILE='DENS.INP',STATUS='UNKNOWN')
*
      READ(52,*)NR
      IF (NR.GE.NPNTS) THEN
       WRITE(*,*) '******    STOP: NR larger than NPNTS '
       STOP
      ENDIF
*
      DO I=0,NR-1
       READ(52,*)RU(I),RH1(I),RH2(I)
      ENDDO
      CLOSE(UNIT=52)
*
* interpolate densities
      DO I=1,NPNTS
        X=R(I)
        BIG=1.D30
        CALL SPLINE(RU,RH1,NR,BIG,BIG,Y2A)
        CALL SPLINT(RU,RH1,Y2A,NR,X,RHO1(I))      
        CALL SPLINE(RU,RH2,NR,BIG,BIG,Y2A)
        CALL SPLINT(RU,RH2,Y2A,NR,X,RHO2(I))      
      ENDDO
      RHO1(0)=RHO1(1)
      RHO2(0)=RHO2(1)
*
      RETURN
      END
*
*=====================================================================
*
      SUBROUTINE OMP_READ(R,DR,NPNTS,UM3Y)
*
* reads densities from file 'OMP.INP': R  x  V(R)  
* First row in OMP.INP = number of rows in input
*
      PARAMETER(NLG=10000)
*
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 R(0:NLG),RU(0:NLG)
      REAL*8 V(0:NLG),UM3Y(0:NLG),VN1(0:NLG),Y2A(0:NLG)
*
      OPEN(UNIT=51,FILE='OMP.INP',STATUS='UNKNOWN')
*
      READ(51,*)NR
      IF (NR.GE.NPNTS) THEN
       WRITE(*,*) '******    STOP: NR larger than NPNTS '
       STOP
      ENDIF
*
      DO I=0,NR-1
       READ(51,*)RU(I),V(I)
      ENDDO
      CLOSE(UNIT=51)
*
* interpolate potential
      DO I=1,NPNTS
        X=R(I)
        BIG=1.D30
        CALL SPLINE(RU,V,NR,BIG,BIG,Y2A)
        CALL SPLINT(RU,V,Y2A,NR,X,UM3Y(I))      
      ENDDO
      UM3Y(0)=UM3Y(1)
*
      RETURN
      END
*
*=====================================================================
*
      REAL*8 FUNCTION TM3Y(S)
      IMPLICIT REAL*8 (A-H,O-Z)
*
*    M3Y effective interation: delta-function treated separately
*
      A3Y=7999.
      B3Y=-2134.
      BETA1=4.
      BETA2=2.5
*
      S1=S*BETA1
      S2=S*BETA2
      TM3Y= A3Y*EXP(-S1)/S1+B3Y*EXP(-S2)/S2
*
      RETURN
      END
*
*=====================================================================
*
      SUBROUTINE TWOFOLD(R,DR,NPNTS,DENS1,DENS2,V)
*
*  Folding of densities:  V = integral DENS1(|r-R|) DENS2(R) d^3 R
*
      PARAMETER(NLG=1000)
*
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 DENS1(0:NLG),DENS2(0:NLG),R(0:NLG),X(0:NLG),
     &       RINT(0:NLG),V(0:NLG),GINT(0:NLG)
*
      PI=3.141597265
      NX=NLG
      NR=NLG
      DX=2./NX
      RJMAX=30.
      DELR=RJMAX/NR
*
      DO I=0,NX
       X(I)=-1.+I*DX
      ENDDO
      DO I=0,NR
       RINT(I)=I*DELR
      ENDDO
*
      DO I=0,NPNTS
        SUM2=0.
        DO J=0,NR           ! integration over radial coordinate
	    SUM1=0.
	    DO K=0,NX         ! integration over polar angle
	      AUX=SQRT(ABS(R(I)**2+RINT(J)**2-2.*R(I)*RINT(J)*X(K)))
	      IFRAC=INT(AUX/DR)
	      AL=1.-(AUX-DR*IFRAC)/DR
	      IF(IFRAC+1.LT.NPNTS.AND.IFRAC.GT.0)
     &           SUM1=SUM1+(AL*DENS1(IFRAC)+(1.-AL)*DENS1(IFRAC+1))
	    ENDDO
	    IFRAC=INT(RINT(J)/DR+1.)
	    AL=1.-(RINT(J)-DR*IFRAC)/DR
	    IF(IFRAC+1.LT.NPNTS.AND.IFRAC.GT.0)
     &           SUM2=SUM2+RINT(J)**2*(AL*DENS2(IFRAC)
     &             +(1.-AL)*DENS2(IFRAC+1))*SUM1
	  ENDDO
	  V(I)=2.*PI*SUM2*DELR*DX
      ENDDO
*
      RETURN
      END
*
*======================================================================
*
      FUNCTION DERIV(R)
*
* Local momentum function
*
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 DERIV
      COMMON/DER/AM2,AUX1
      COMMON/COU/Z1,Z2,A1,A2,RC,RC2,AUX6
      COMMON/POT2/V0,R0,AA,VS0,RS0,AAS,AM,A13,ECM
      COMMON/SPIN2/AJ,ALL,ALS
*
C      PRINT*,'ENTREI',R,ALL,AUX1,VCOU(R)
      DERIV=ALL/(R*R)-AUX1*(ECM-VCOU(R)-VNR(R))
*
      RETURN
      END
*
*======================================================================
*
      SUBROUTINE CONTWAVE(PSI)
*
* Solves continuum state Schroedinger equation using a Runge-Kutta method
*
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER(NLG=10000)
      COMMON/SNBAR/SNBAR,PSHIFT
      COMMON/COU/Z1,Z2,A1,A2,RC,RC2,AUX6
      COMMON/INTE1/DR,RI
      COMMON/INTE2/NAX
      COMMON/ET/ETA,XK
      COMMON/POT2/V0,R0,AA,VS0,RS0,AAS,AM,A13,ECM
      COMMON/SPIN1/L
      DIMENSION F(71),FP(71),G(71),GP(71),CNN(NLG)
      COMPLEX*16 HP,HM,DHP,DHM,DLG,ANORM,SNUC
      COMPLEX*16 PSI(NLG)
      COMPLEX*16 YY(2)
*
      DR1=DR/10.
      N=NAX+1
      PSI(1)=DCMPLX(0.D0,0.D0)
      YY(1)=DCMPLX(0.D0,0.D0)
      YY(2)=DCMPLX(1.D0,0.D0)
      R=1.D-10
      CNN(1)=0.D0
*
* begin integration up to R=RMAX
*
      DO 1000 I=2,N
        CNN(I)=CNN(I-1)
        CALL RK4(YY,2,R,DR)
        CHN=ABS(YY(1))
*
* Renormalization
        IF(CHN.GT.1.D30)THEN
          CNN(I)=CNN(I)+DLOG(CHN)
          YY(1)=YY(1)/CHN
          YY(2)=YY(2)/CHN
        END IF
*
        PSI(I)=YY(1)
1000  CONTINUE
*
* End integration and match with Coulomb waves at r=RMAX
*
      RMAX=DR*DBLE(NAX)
      LP=L+1
      ROM=RMAX*XK
      ACCUR=1.D0
      STEP=100.D0
      CALL COULOMB(ROM,ETA,L,L,F,FP,G,GP,ACCUR,STEP)
      DEVI=R-RMAX
      IF (DABS(DEVI).GT.DR1) WRITE(6,707)R,RMAX,DEVI
707   FORMAT(/,' PROBLEM!! RMATCH (=',f8.4,' fm) different 
     & than RMAX (=',f8.4,' fm) !! Delta R=',g10.2//)
*
* S-matrix
*
      HP=DCMPLX(G(LP),F(LP))
      HM=DCONJG(HP)
      DHP=DCMPLX(GP(LP),FP(LP))
      DHM=DCONJG(DHP)
      DLG=YY(2)/YY(1)
      SNUC=(XK*DHM-DLG*HM)/(XK*DHP-DLG*HP)
*
* normalizes wavefunction
      ANORM=0.5*(HM-SNUC*HP)*DCMPLX(CEXP(CMPLX(0.,-1.57)))/YY(1)
      ANORM=SQRT(2./3.141519)*ANORM
      DO 2000 I=2,N
        RENO=EXP(CNN(N)-CNN(I))
        PSI(I)=PSI(I)*ANORM/RENO
2000  CONTINUE
*
* phase shift
*
      SNBAR=CDABS(SNUC)
      PSHIFT=DATAN2(DIMAG(SNUC),DBLE(REAL(SNUC)))/2.
*
      RETURN
      END
*
*======================================================================
*
      SUBROUTINE DERIVS(X,Y,DYDX)
*
* Wavefunction (Y1) and its derivative at X (Y2)
*
      IMPLICIT REAL*8(A-H,O-Z)
      COMPLEX*16 Y(2),DYDX(2)
*
      DYDX(1)=Y(2)
      DYDX(2)=Y(1)*DERIV(X)
*
      RETURN
      END
*
*=====================================================================
*
      SUBROUTINE RK4(Y,N,X,H)
*
* Runge-Kutta: iterates to get Y(X_(i+1)) from Y(X_i)
*
      PARAMETER (NMAX=10)
      IMPLICIT REAL*8(A-H,O-Z)
      COMPLEX*16 Y,DYDX,YT,DYT,DYM
      DIMENSION Y(N),DYDX(NMAX),YT(NMAX),DYT(NMAX),DYM(NMAX)
*
      HH=H*0.5
      H6=H/6.
      XH=X+HH
*
      CALL DERIVS(X,Y,DYDX)
      DO 11 I=1,N
        YT(I)=Y(I)+HH*DYDX(I)
 11   CONTINUE
      CALL DERIVS(XH,YT,DYT)
      DO 12 I=1,N
        YT(I)=Y(I)+HH*DYT(I)
 12   CONTINUE
      CALL DERIVS(XH,YT,DYM)
      DO 13 I=1,N
        YT(I)=Y(I)+H*DYM(I)
        DYM(I)=DYT(I)+DYM(I)
 13   CONTINUE
      CALL DERIVS(X+H,YT,DYT)
      DO 14 I=1,N
        Y(I)=Y(I)+H6*(DYDX(I)+DYT(I)+2.*DYM(I))
 14   CONTINUE
      X=X+H
*
      RETURN
      END
*
*=====================================================================
*
      SUBROUTINE COULOMB(RHO,ETA,MINL,MAXL,FC,FCP,GC,GCP,ACCUR,STEP)
*
*  Coulomb waves
*  =============
*  Coulomb wavefunctions calculated at R=RHO by the continued fraction 
*  method of STeed  
*  MINL,MAXL are actual L-values
*  see Barnett Feng Steed and Goldfarb Computer Physics Commun 1974
*
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*4 K,K1,K2,K3,K4,M1,M2,M3,M4
      DIMENSION FC(71),FCP(71),GC(71),GCP(71)
*
      PACE=STEP
      ACC=ACCUR
      IF(PACE.LT.100.0)PACE=100.0
      IF(ACC.LT.1.0E-15.OR.ACC.GT.1.0E-6)ACC=1.0E-6
      R=RHO
      KTR=1
      LMAX=MAXL
      LMIN1=MINL+1
      XLL1=DBLE(MINL*LMIN1)
      ETA2=ETA*ETA
      TURN=ETA+DSQRT(ETA2+XLL1)
      IF(R.LT.TURN.AND.DABS(ETA).GE.1.0E-6)KTR=-1
      KTRP=KTR
      GO TO 2
1     R=TURN
      TF=F
      TFP=FP
      LMAX=MINL
      KTRP=1
2     ETAR=ETA*R
      RHO2=R*R
      PL=DBLE(LMAX+1)
      PMX=PL+0.5
*
*  continued fraction for FP(MAXL)/F(MAXL): XL is F, XLPRIME is FP
      FP=ETA/PL+PL/R
      DK=ETAR*2.0
      DEL=0.0
      D=0.0
      F=1.0
      K=(PL*PL-PL+ETAR)*(2.0*PL-1.0)
      IF(PL*PL+PL+ETAR.NE.0.0)GO TO 3
      R=R+1.0E-6
      GO TO 2
3     H=(PL*PL+ETA2)*(1.0-PL*PL)*RHO2
      K=K+DK+PL*PL*6.0
      D=1.0/(D*H+K)
      DEL=DEL*(D*K-1.0)
      IF(PL.LT.PMX)DEL=-R*(PL*PL+ETA2)*(PL+1.0)*D/PL
      PL=PL+1.0
      FP=FP+DEL
      IF(D.LT.0.0)F=-F
      IF(PL.GT.20000.)GO TO 11
      IF(DABS(DEL/FP).GE.ACC)GO TO 3
      FP=F*FP
      IF(LMAX.EQ.MINL)GO TO 5
      FC(LMAX+1)=F
      FCP(LMAX+1)=FP
*
* downward recursion to MINL for F and FP,arrays GC,GCP are STORAGE
      L=LMAX
      DO 4 LP=LMIN1,LMAX
        PL=DBLE(L)
        GC(L+1)=ETA/PL+PL/R
        GCP(L+1)=DSQRT(ETA2+PL*PL)/PL
        FC(L)=(GC(L+1)*FC(L+1)+FCP(L+1))/GCP(L+1)
        FCP(L)=GC(L+1)*FC(L)-GCP(L+1)*FC(L+1)
4     L=L-1
      F=FC(LMIN1)
      FP=FCP(LMIN1)
5     IF(KTRP.EQ.-1)GO TO 1
*
*    repeat for R=TURN if RHO lt TURN
*    now obtain P+I.Q for MINL from continued fraction (32)
*    real arithmetic to facilitate conversion using real*8
      P=0.0
      Q=R-ETA
      PL=0.0
      AR=-(ETA2+XLL1)
      AI=ETA
      BR=2.0*Q
      BI=2.0
      WI=2.0*ETA
      DR=BR/(BR*BR+BI*BI)
      DI=-BI/(BR*BR+BI*BI)
      DP=-(AR*DI+AI*DR)
      DQ=(AR*DR-AI*DI)
6     P=P+DP
      Q=Q+DQ
      PL=PL+2.0
      AR=AR+PL
      AI=AI+WI
      BI=BI+2.0
      D=AR*DR-AI*DI+BR
      DI=AI*DR+AR*DI+BI
      T=1.0/(D*D+DI*DI)
      DR=T*D
      DI=-T*DI
      H=BR*DR-BI*DI-1.0
      K=BI*DR+BR*DI
      T=DP*H-DQ*K
      DQ=DP*K+DQ*H
      DP=T
      IF(PL.GT.46000.)GO TO 11
      IF(DABS(DP)+DABS(DQ).GE.(DABS(P)+DABS(Q))*ACC)GO TO 6
      P=P/R
      Q=Q/R
*
*  solve for FP,G,GP and normalise F at L=MINL
      G=(FP-F*P)/Q
      GP =P*G-Q*F
      W=1.0/DSQRT(FP*G-F*GP)
      G=W*G
      GP=W*GP
      IF(KTR.EQ.1)GO TO 8
      F=TF
      FP=TFP
      LMAX=MAXL
*
* Runge-Kutta integration of G(MINL) and GP(MINL) inwards from turn
*              see Fox and Mayers 1968, pg 202
      IF(RHO.LT.0.2*TURN)PACE=999.0
      R3=1.0/3.0
      H=(RHO-TURN)/(PACE+1.0)
      H2=0.5*H
      I2=(PACE+0.001)
      ETAH=ETA*H
      H2LL=H2*XLL1
      S=(ETAH+H2LL/R)/R-H2
7     RH2=R+H2
      T=(ETAH+H2LL/RH2)/RH2-H2
      K1=H2*GP
      M1=S*G
      K2=H2*(GP+M1)
      M2=T*(G+K1)
      K3=H*(GP+M2)
      M3=T*(G+K2)
      M3=M3+M3
      K4=H2*(GP+M3)
      RH=R+H
      S=(ETAH+H2LL/RH)/RH-H2
      M4=S*(G+K3)
      G=G+(K1+K2+K2+K3+K4)*R3
      GP=GP+(M1+M2+M2+M3+M4)*R3
      R=RH
      I2=I2-1
      IF(I2.GE.0)GO TO 7
      W=1.0/(FP*G-F*GP)
*
*  upward recursion from GC(MINL) and GCP(MINL),stored values are R,S
*  renormalise FC,FCP for each L-value
8     GC(LMIN1)=G
      GCP(LMIN1)=GP
      IF(LMAX.EQ.MINL)GO TO 10
      DO 9 L=LMIN1,LMAX
        T=GC(L+1)
        GC(L+1)=(GC(L)*GC(L+1)-GCP(L))/GCP(L+1)
        GCP(L+1)=GC(L)*GCP(L+1)-GC(L+1)*T
        FC(L+1)=W*FC(L+1)
9       FCP(L+1)=W*FCP(L+1)
      FC(LMIN1)=FC(LMIN1)*W
      FCP(LMIN1)=FCP(LMIN1)*W
      RETURN
*
10    FC(LMIN1)=W*F
      FCP(LMIN1)=W*FP
      RETURN
*
11    W=0.0
      G=0.0
      GP=0.0
*
      GO TO 8
      END
*
*=====================================================================
*
      REAL*8 FUNCTION CLEBSH(AJ1,AJ2,AJ3,AM1, AM2, AM3)
*
*  Clebsh-Gordan coefficients 
*
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 IV(0:130)
*
      COMMON /GF1 / IV
*
      CLEBSH=IV(NINT(AJ1-AJ2+AM3))*SQRT(2.*AJ3+1.)*
     &       THREEJ(AJ1,AJ2,AJ3,AM1,AM2,-AM3)
*
      RETURN
      END
*
*=====================================================================
*
      REAL*8 FUNCTION THREEJ (AJ1, AJ2, AJ3,
     .                        AM1, AM2, AM3)
*
*  THREEJ is a Wigner 3J-SYMBOL 
*
      IMPLICIT REAL*8 (A-H,O-Z)
*
      J1=NINT(2.*AJ1+1.)
      J2=NINT(2.*AJ2+1.)
      J3=NINT(2.*AJ3+1.)
      M1=NINT(2.*AM1)
      M2=NINT(2.*AM2)
      M3=NINT(2.*AM3)
      THREEJ=S3J(J1,J2,J3,M1,M2,M3)
*
      RETURN
      END
*
*=====================================================================
*
      REAL*8 FUNCTION SIXJ (AJ1, AJ2, AJ3,
     &                      AI1, AI2, AI3)
*
*  SIXJ is a Wigner 6J-SYMBOL 
*
      IMPLICIT REAL*8 (A-H,O-Z)
*
      J1=NINT(2.*AJ1+1.)
      J2=NINT(2.*AJ2+1.)
      J3=NINT(2.*AJ3+1.)
      I1=NINT(2.*AI1+1.)
      I2=NINT(2.*AI2+1.)
      I3=NINT(2.*AI3+1.)
      SIXJ=S6J(J1,J2,J3,I1,I2,I3)
*
      RETURN
      END
*
*======================================================================
*
      REAL*8 FUNCTION S3J (J1, J2, J3,
     &                     M1, M2, M3)
*
*  S3J is a 3J-SYMBOL. All J's are given as 2J+1, all M's as 2M        
*
      IMPLICIT REAL*8 (A-H,O-Z)
*
      LOGICAL SET
      REAL*8 F(99), SF(99)
      SAVE SET, F, SF
      DATA SET / .TRUE. /
*
      IF (SET) THEN
         F(1) = 1.0D0
         SF(1) = 1.0D0
         DO 10 K = 3, 99, 2
            F(K) = 0.5D0*(K-1)*F(K-2)
            SF(K) = SQRT(F(K))
   10    CONTINUE
         SET = .FALSE.
      END IF

      S3J = 0.0D0
      K1 = J1 + M2 - J3
      K2 = J2 - M1 - J3
      L1 = J1 - M1
      L2 = J2 + M2
      L3 = J1 + J2 - J3
      K = MAX (0, K1, K2)
      L = MIN (L1, L2, L3)
      IF (K.GT.L) RETURN
      TERM = 1.0D0/(F(K+1)*F(K-K1+1)*F(K-K2+1)
     &	*F(L1-K)*F(L2-K)*F(L3-K))
      IF (MOD(K1-K2+K,4).NE.0D0) TERM = -TERM
      SUM = TERM
      DO 20 K = K+2, L, 2
         TERM = TERM*(K-L1-1)*(L2-K+1)*(L3-K+1)/
     &   	   (K*(K-K1)*(K-K2))
         SUM = SUM + TERM
   20 CONTINUE
      S3J = SUM*SF(L3)*SF(J2+J3-J1)*SF(J3+J1-J2)/SF(J1+J2+J3)
     &      *SF(J1+M1)*SF(J1-M1)*SF(J2+M2)*SF(J2-M2)
     &     *SF(J3+M3)*SF(J3-M3)
*
      RETURN
      END
*
*======================================================================
*
      REAL*8 FUNCTION S6J (J1, J2, J3,
     .                     I1, I2, I3)
*
*  S6J is a 6J-SYMBOL. All J's are given as 2J+1, all M's as 2M        
*
      IMPLICIT REAL*8 (A-H,O-Z)
*
      SJ=0.0 
      DO 22 M1 = -J1+1,J1-1,2
      DO 22 M2 = -J2+1,J2-1,2
      DO 22 M3 = -J3+1,J3-1,2
      DO 22 M4 = -I2+1,I2-1,2
      DO 22 M5 = -I3+1,I3-1,2
      DO 22 M6 = -I1+1,I1-1,2
      IF(M1+M2.EQ.-M3.AND.M1+M4.EQ.M5.AND.M2-M6
     &    	.EQ.-M5.AND.M4-M6.EQ.M3)
     . THEN
      F = 1-2*MOD((I1+I2+I3-3+M6+M4+M5)/2,2)
      SJ=SJ+F*S3J(J1,J2,J3,M1,M2,M3)
     .     *S3J(J1,I2,I3,M1,M4,-M5)
     .     *S3J(I1,J2,I3,-M6,M2,M5)
     .     *S3J(I1,I2,J3,M6,-M4,M3)
      ELSE
      END IF
   22 CONTINUE
      S6J=SJ
*
      RETURN
      END
*
*=====================================================================
*      
      SUBROUTINE GFV(IE)
*
*  Calculates sign and factorials  of integers and half int.
*
*     iv(n)  =  (-1)**n
*     fak(n) =  n!
*     fad(n) =  n!!
*
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 IV(0:130),FAK(0:130),FAD(0:130)
      COMMON /GF1 / IV
      COMMON /GF2 / FAK
      COMMON /GF3 / FAD
*
      IF (IE.GT.130) THEN
       WRITE(12,*) '******    STOP IN GFV: IE LARGER 130 '
       STOP
      ENDIF
      IV(0)  = +1
      FAK(0) =  1.D0
      FAD(0)=1.
      FAD(1)=1.
      DO 10 I=1,IE
         IV(I)  = -IV(I-1)
10       FAK(I) = I*FAK(I-1)
      DO 11 I=3,IE,2
         FAD(I) = I*FAD(I-2)
11    CONTINUE 
      DO 12 I=2,IE,2
         FAD(I) = I*FAD(I-2)
12    CONTINUE 
*
      RETURN
	END
*
*=====================================================================
*      
      SUBROUTINE DERIVATIVE(WF,WF1,WF2,DX,NPNTS)
*
* Calculates an array of first (WF1) and second (WF2) derivatives of 
* the array function WF(N).
*
      PARAMETER(NLG=10000)
*
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 A(6,6),B(6,6),WF(0:NLG),WF1(0:NLG),WF2(0:NLG)
*
      DS=120.0D0*DX
* 
      A(1,1)=-274.0D0/DS
      A(1,2)=600.0D0/DS
      A(1,3)=-600.0D0/DS
      A(1,4)=400.0D0/DS
      A(1,5)=-150.0D0/DS
      A(1,6)=24.0D0/DS
      A(2,1)=-24.0D0/DS
      A(2,2)=-130.0D0/DS
      A(2,3)=240.0D0/DS
      A(2,4)=-120.0D0/DS
      A(2,5)=40.0D0/DS
      A(2,6)=-6.0D0/DS
      A(3,1)=6.0D0/DS
      A(3,2)=-60.0D0/DS
      A(3,3)=-40.0D0/DS
      A(3,4)=120.0D0/DS
      A(3,5)=-30.0D0/DS
      A(3,6)=4.0D0/DS
      A(4,1)=-4.0D0/DS
      A(4,2)=30.0D0/DS
      A(4,3)=-120.0D0/DS
      A(4,4)=40.0D0/DS
      A(4,5)=60.0D0/DS
      A(4,6)=-6.0D0/DS
      A(5,1)=6.0D0/DS
      A(5,2)=-40.0D0/DS
      A(5,3)=120.0D0/DS
      A(5,4)=-240.0D0/DS
      A(5,5)=130.0D0/DS
      A(5,6)=24.0D0/DS
      A(6,1)=-24.0D0/DS
      A(6,2)=150.0D0/DS
      A(6,3)=-400.0D0/DS
      A(6,4)=600.0D0/DS
      A(6,5)=-600.0D0/DS
      A(6,6)=274.0D0/DS
*
      DS=60.0D0*DX**2
*
      B(1,1)=225.0D0/DS
      B(1,2)=-770.0D0/DS
      B(1,3)=1070.0D0/DS
      B(1,4)=-780.0D0/DS
      B(1,5)=305.0D0/DS
      B(1,6)=-50.0D0/DS
      B(2,1)=50.0D0/DS
      B(2,2)=-75.0D0/DS
      B(2,3)=-20.0D0/DS
      B(2,4)=70.0D0/DS
      B(2,5)=-30.0D0/DS
      B(2,6)=5.0D0/DS
      B(3,1)=-5.0D0/DS
      B(3,2)=80.0D0/DS
      B(3,3)=-150.0D0/DS
      B(3,4)=80.0D0/DS
      B(3,5)=-5.0D0/DS
      B(3,6)=0.0D0
      B(4,1)=0.0D0
      B(4,2)=-5.0D0/DS
      B(4,3)=80.0D0/DS
      B(4,4)=-150.0D0/DS
      B(4,5)=80.0D0/DS
      B(4,6)=-5.0D0/DS
      B(5,1)=5.0D0/DS
      B(5,2)=-30.0D0/DS
      B(5,3)=70.0D0/DS
      B(5,4)=-20.0D0/DS
      B(5,5)=-75.0D0/DS
      B(5,6)=50.0D0/DS
      B(6,1)=-50.0D0/DS
      B(6,2)=305.0D0/DS
      B(6,3)=-780.0D0/DS
      B(6,4)=1070.0D0/DS
      B(6,5)=-770.0D0/DS
      B(6,6)=225.0D0/DS
*
      DO 2 IR=3,NPNTS-3
	WF1(IR)=A(3,1)*WF(IR-2)+A(3,2)*WF(IR-1)+A(3,3)*WF(IR)
     &    +A(3,4)*WF(IR+1)+A(3,5)*WF(IR+2)+A(3,6)*WF(IR+3)
	WF2(IR)=B(3,1)*WF(IR-2)+B(3,2)*WF(IR-1)+B(3,3)*WF(IR)
     &    +B(3,4)*WF(IR+1)+B(3,5)*WF(IR+2)+B(3,6)*WF(IR+3)
2     CONTINUE
*
      IR=1
      WF1(IR)=A(1,1)*WF(IR)+A(1,2)*WF(IR+1)+A(1,3)*WF(IR+2)
     &    +A(1,4)*WF(IR+3)+A(1,5)*WF(IR+4)+A(1,6)*WF(IR+5)
      WF2(IR)=B(1,1)*WF(IR)+B(1,2)*WF(IR+1)+B(1,3)*WF(IR+2)
     &    +B(1,4)*WF(IR+3)+B(1,5)*WF(IR+4)+B(1,6)*WF(IR+5)
*
      IR=2
      WF1(IR)=A(2,1)*WF(IR-1)+A(2,2)*WF(IR)+A(2,3)*WF(IR+1)
     &    +A(2,4)*WF(IR+2)+A(2,5)*WF(IR+3)+A(2,6)*WF(IR+4)
      WF2(IR)=B(2,1)*WF(IR-1)+B(2,2)*WF(IR)+B(2,3)*WF(IR+1)
     &    +B(2,4)*WF(IR+2)+B(2,5)*WF(IR+3)+B(2,6)*WF(IR+4)
*
      IR=NPNTS-2
      WF1(IR)=A(4,1)*WF(IR-3)+A(4,2)*WF(IR-2)+A(4,3)*WF(IR-1)
     &    +A(4,4)*WF(IR)+A(4,5)*WF(IR+1)+A(4,6)*WF(IR+2)
      WF2(IR)=B(4,1)*WF(IR-3)+B(4,2)*WF(IR-2)+B(4,3)*WF(IR-1)
     &    +B(4,4)*WF(IR)+B(4,5)*WF(IR+1)+B(4,6)*WF(IR+2)
*
      IR=NPNTS-1
      WF1(IR)=A(5,1)*WF(IR-4)+A(5,2)*WF(IR-3)+A(5,3)*WF(IR-2)
     &    +A(5,4)*WF(IR-1)+A(5,5)*WF(IR)+A(5,6)*WF(IR+1)
      WF2(IR)=B(5,1)*WF(IR-4)+B(5,2)*WF(IR-3)+B(5,3)*WF(IR-2)
     &    +B(5,4)*WF(IR-1)+B(5,5)*WF(IR)+B(5,6)*WF(IR+1)
*
      IR=NPNTS
      WF1(IR)=A(6,1)*WF(IR-5)+A(6,2)*WF(IR-4)+A(6,3)*WF(IR-3)
     &    +A(6,4)*WF(IR-2)+A(6,5)*WF(IR-1)+A(6,6)*WF(IR)
      WF2(IR)=B(6,1)*WF(IR-5)+B(6,2)*WF(IR-4)+B(6,3)*WF(IR-3)
     &    +B(6,4)*WF(IR-2)+B(6,5)*WF(IR-1)+B(6,6)*WF(IR)
*
      RETURN
      END
*
*=====================================================================
*      
      SUBROUTINE SPLINE(X,Y,N,YP1,YPN,Y2)
*
* Given arrays X[1..N] and Y[1..N], with X1 < X2 < ... < XN, and given  
* values of YP1 and YPN for the first derivative of the interpolating 
* function at points 1 and N, respectively, this routine returns an array 
* Y2[1..N] that contains the second derivatives of the interpolating function 
* at the tabulated points X. If YP1 an/or YPN are equal to 1 x 10^30 or 
* larger, the routine is signaled to set the corresponding boundary condition 
* for a natural spline, with zero second derivative on that bounday.
*
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 X(0:N),Y(0:N),Y2(0:N)
      PARAMETER (NMAX=10000)
      REAL*8 U(0:NMAX)
*
* The lower boundary condition is set either to be "natural" or else to 
* have a specified first derivative.
      IF (YP1.GT..99D30) THEN
        Y2(1)=0.
        U(1)=0.
      ELSE
        Y2(1)=-0.5
        U(1)=(3./(X(2)-X(1)))*((Y(2)-Y(1))/(X(2)-X(1))-YP1)
      ENDIF
*
* This is the decomposition loop of the tridiagonal algorithm. Y2 and U are 
* used for temporary storage of the decomposed factors.
      DO 11 I=2,N-1
        SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
        P=SIG*Y2(I-1)+2.
        Y2(I)=(SIG-1.)/P
        U(I)=(6.*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1))/
     &      (X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
11    CONTINUE
*
* The lower boundary condition is set either to be "natural" or else to have 
* a specified first derivative.
      IF (YPN.GT..99D30) THEN
        QN=0.
        UN=0.
      ELSE
        QN=0.5
        UN=(3./(X(N)-X(N-1)))*(YPN-(Y(N)-Y(N-1))/(X(N)-X(N-1)))
      ENDIF
      Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.)
*
* This is the backsubstitution loop of the tridiagonal algorithm.
      DO 12 K=N-1,1,-1
        Y2(K)=Y2(K)*Y2(K+1)+U(K)
12    CONTINUE
*
      RETURN
      END
*
*=====================================================================
*      
      SUBROUTINE SPLINT(XA,YA,Y2A,N,X,Y)
*
* Given arrays XA[1..N] and YA[1..N], which tabulate a function (with the 
* XA's in order), and given the array Y2A[1..N], which is the output from 
* spline  program above, and given a value of X this routine returns a 
* cubic-spline interpolated value Y.
*
      IMPLICIT REAL*8(A-H,O-Z)
      REAL*8 XA(0:N),Y2A(0:N),YA(0:N)
*
      KLO=1
      KHI=N
*
* We will find the right place in the table by means of bisection.
* This is optimal if sequential calls to this routine are at random values 
* of X. If sequential calls are in order, and closely spaced, one would do 
* better to store the previous values of KLO and KHI and test if they remain 
* appropriate on the next call.
1     IF (KHI-KLO.GT.1) THEN
        K=(KHI+KLO)/2
        IF(XA(K).GT.X)THEN
          KHI=K
        ELSE
          KLO=K
        ENDIF
      GOTO 1
      ENDIF
*
* KLO and KHI now bracket the input value of X.
      H=XA(KHI)-XA(KLO)
*
* This error can occur only if two input XA's are (to within roundoff) identical
      IF (H.EQ.0.) THEN
       PRINT*,'******    Bad XA input in SPLINT'
       STOP
      ENDIF
      A=(XA(KHI)-X)/H
      B=(X-XA(KLO))/H
      Y=A*YA(KLO)+B*YA(KHI)+((A**3-A)*Y2A(KLO)+
     &    	(B**3-B)*Y2A(KHI))*(H**2)/6.
*
      RETURN
      END
*
*=====================================================================
*
      SUBROUTINE SKIPCOM(IUNIT)
*
* allows comments in input files
*
      CHARACTER*1 ICOM
*
1     READ(IUNIT,'(A1)')ICOM
      IF(ICOM.EQ.'*')GOTO 1
      BACKSPACE IUNIT
*
      RETURN
      END
**********************************************************************